"""
Django Ledger created by Miguel Sanda <msanda@arrobalytics.com>.
Copyright© EDMA Group Inc licensed under the GPLv3 Agreement.
"""

default_app_config = 'django_ledger.apps.DjangoLedgerConfig'

"""Django Ledger"""
__version__ = '0.8.0'
__license__ = 'GPLv3 License'

__author__ = 'Miguel Sanda'
__email__ = 'msanda@arrobalytics.com'

__url__ = 'https://www.djangoledger.com'
