<form id="djetler-set-entity-form-{{ form_id }}"
      action="{% url 'django_ledger:entity-set-default' %}"
      method="post">
    {% csrf_token %}
    <div class="field">
        <div class="control">
            <div class="select is-inf is-small">
                {{ default_entity_form.entity_model }}
            </div>
        </div>
    </div>
</form>