# Generated by Django 5.1.5 on 2025-01-17 00:11

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('django_ledger', '0018_transactionmodel_cleared_transactionmodel_reconciled_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='transactionmodel',
            name='amount',
            field=models.DecimalField(decimal_places=2, default=0.0, help_text='Amount of the transaction.',
                                      max_digits=20, validators=[django.core.validators.MinValueValidator(0)],
                                      verbose_name='Amount'),
        ),
        migrations.AlterField(
            model_name='transactionmodel',
            name='description',
            field=models.CharField(blank=True,
                                   help_text='A description to be included with this individual transaction.',
                                   max_length=100, null=True, verbose_name='Transaction Description'),
        ),
        migrations.AlterField(
            model_name='transactionmodel',
            name='tx_type',
            field=models.CharField(choices=[('credit', 'Credit'), ('debit', 'Debit')], max_length=10,
                                   verbose_name='Transaction Type'),
        ),
    ]
