{% load i18n %}
{% load django_ledger %}

<div class="card" id="djl-vendor-card-widget">
    <div class="card-header">
        <h2 class="card-header-title has-text-weight-light is-size-3">
            <span class="icon is-large">
                    {% icon 'bi:person-lines-fill' 36 %}</span>
            {% trans 'Vendor Info' %}</h2>
    </div>
    <div class="card-content">
        <div class="content">
            <h1 class="has-text-weight-bold">
                {{ vendor.vendor_name }}
            </h1>
            <p>
                {% if vendor.address_1 %}{{ vendor.address_1 }}<br>{% endif %}
                {% if vendor.address_2 %}{{ vendor.address_2 }}<br>{% endif %}
                {% if vendor.get_cszc %}{{ vendor.get_cszc }}<br>{% endif %}
                {% if vendor.phone %}{{ vendor.phone }}<br>{% endif %}
                {% if vendor.email %}{{ vendor.email }}<br>{% endif %}
                {% if vendor.website %}{{ vendor.website }}<br>{% endif %}
            </p>
        </div>
    </div>
    <footer class="card-footer">
        <a href="{% url 'django_ledger:vendor-update' vendor_pk=vendor.uuid entity_slug=view.kwargs.entity_slug %}"
           class="card-footer-item has-text-primary has-text-centered">{% trans 'Edit' %}</a>
    </footer>
</div>
