{% extends 'django_ledger/layouts/base.html' %}

{% block content %}
    <section class="section">
        <div class="container">
            <div class="columns">
                <div class="column">
                    {% block view_content %}{% endblock %}
                </div>
            </div>
        </div>
    </section>
{% endblock content %}