# Generated by Django 5.1.6 on 2025-02-13 01:40

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('django_ledger', '0019_alter_transactionmodel_amount_and_more'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='bankaccountmodel',
            name='django_ledg_cash_ac_59a8af_idx',
        ),
        migrations.RenameField(
            model_name='bankaccountmodel',
            old_name='cash_account',
            new_name='account_model',
        ),
        migrations.AlterUniqueTogether(
            name='bankaccountmodel',
            unique_together={('entity_model', 'account_model', 'account_number', 'routing_number'),
                             ('entity_model', 'account_number')},
        ),
        migrations.AlterField(
            model_name='bankaccountmodel',
            name='account_type',
            field=models.CharField(
                choices=[('checking', 'Checking'), ('savings', 'Savings'), ('credit_card', 'Credit Card'),
                         ('mortgage', 'Mortgage')], default='checking', max_length=20, verbose_name='Account Type'),
        ),
        migrations.AlterField(
            model_name='vendormodel',
            name='account_type',
            field=models.CharField(
                choices=[('checking', 'Checking'), ('savings', 'Savings'), ('credit_card', 'Credit Card'),
                         ('mortgage', 'Mortgage')], default='checking', max_length=20, verbose_name='Account Type'),
        ),
        migrations.AddIndex(
            model_name='bankaccountmodel',
            index=models.Index(fields=['account_model'], name='django_ledg_account_ed900d_idx'),
        ),
    ]
