{% load i18n %}

<form action="{{ form_path }}" method="get">

    <div class="field is-horizontal">

        <div class="field-label is-small">
            <label for="{{ activity_form.activity.id_for_label }}"
                   class="label">{{ activity_form.activity.label }}
            </label>
        </div>

        <div class="field-body">
            <div class="field has-addons">
                <div class="control is-expanded">
                    <div class="select is-small is-fullwidth">{{ activity_form.activity }}</div>
                </div>
                <div class="control">
                    <button type="submit" class="button is-primary is-small">Choose</button>
                </div>
            </div>
        </div>

    </div>

</form>
