# Generated by Django 4.2.2 on 2023-10-08 03:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_ledger', '0011_importjobmodel_alter_accountmodel_role_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='stagedtransactionmodel',
            name='activity',
            field=models.CharField(blank=True, choices=[('Operating', (('op', 'Operating'),)), ('Investing', (('inv_ppe', 'Purchase/Disposition of PPE'), ('inv_securities', 'Purchase/Disposition of Securities'), ('inv', 'Investing Activity Other'))), ('Financing', (('fin_std', 'Payoff of Short Term Debt'), ('fin_ltd', 'Payoff of Long Term Debt'), ('fin_equity', 'Issuance of Common Stock, Preferred Stock or Capital Contribution'), ('fin_dividends', 'Dividends or Distributions to Shareholders'), ('fin', 'Financing Activity Other')))], max_length=20, null=True, verbose_name='Proposed Activity'),
        ),
    ]
