{% extends 'django_ledger/layouts/content_layout_2.html' %}
{% load i18n %}
{% load static %}
{% load django_ledger %}

{% block view_content %}
    <div class="columns is-centered">
        <div class="column is-6">
            <form method="post">
                {% csrf_token %}
                <div class="card">
                    <div class="card-content has-text-centered">
                        <p class="title has-text-weight-light">{{ je_model.get_delete_message }}</p>
                    </div>
                    <div class="card-content has-text-centered">
                        <a href="{{ je_model.get_journal_entry_list_url }}"
                           class="button is-primary">{% trans 'Go Back' %}</a>
                        <button type="submit" class="button is-danger">{% trans 'Delete' %}</button>
                    </div>
                </div>
            </form>
        </div>
    </div>


{% endblock %}




