{% load django_ledger %}
{% load i18n %}

<div class="box content djetler-box-green">
    <h2 class="has-text-centered has-text-weight-light">{% trans 'Assets' %}</h2>
    <h1 class="has-text-centered">
        {% currency_symbol %}{{ tx_digest.group_balance.GROUP_ASSETS | currency_format }}</h1>
    <h2 class="has-text-centered has-text-weight-light">{% trans 'Liabilities' %}</h2>
    <h1 class="has-text-centered">
        {% currency_symbol %}{{ tx_digest.group_balance.GROUP_LIABILITIES | currency_format }}</h1>
    <h2 class="has-text-centered has-text-weight-light">{% trans 'Equity' %}</h2>
    <h1 class="has-text-centered">
        {% currency_symbol %}{{ tx_digest.group_balance.GROUP_EQUITY | currency_format }}</h1>
    <h4 class="has-text-centered has-text-weight-light">{% trans 'Cash' %}</h4>
    <h3 class="has-text-centered">
        {% currency_symbol %}{{ tx_digest.role_balance.ASSET_CA_CASH | currency_format }}</h3>
</div>
