{% extends 'django_ledger/layouts/content_layout_1.html' %}
{% load i18n %}
{% load static %}
{% load django_ledger %}

{% block view_content %}
    <div class="box">
        <div class="columns is-multiline">
            <div class="column is-12 has-text-centered">
                <h1 class="is-size-3">{% trans 'CoA Account List' %}</h1>
                <h2 class="is-size-1 has-text-info">{{ coa_model.name }}</h2>
                <p class="is-italic has-text-danger">({{ coa_model.slug }})</p>
            </div>
            <div class="column is-12">
                {% accounts_table accounts %}
            </div>
            <div class="column is-12 has-text-centered">
                <a href="{{ coa_model.get_create_coa_account_url }}"
                   class="button is-success">{% trans 'Create Account' %}</a>
                <a href="{{ coa_model.get_coa_list_url }}"
                   class="button is-dark">{% trans 'Back to CoA List' %}</a>
            </div>
        </div>
    </div>
{% endblock %}
