# Generated by Django 4.2 on 2023-04-20 20:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_ledger', '0006_alter_journalentrymodel_je_number_and_more'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='customermodel',
            name='django_ledg_custome_16f95a_idx',
        ),
        migrations.AlterUniqueTogether(
            name='accountmodel',
            unique_together={('coa_model', 'code')},
        ),
        migrations.AddField(
            model_name='accountmodel',
            name='role_default',
            field=models.BooleanField(blank=True, null=True, verbose_name='Coa Role Default Account'),
        ),
        migrations.AlterField(
            model_name='accountmodel',
            name='role',
            field=models.CharField(choices=[('Assets', (('asset_ca_cash', 'Current Asset'), ('asset_ca_mkt_sec', 'Marketable Securities'), ('asset_ca_recv', 'Receivables'), ('asset_ca_inv', 'Inventory'), ('asset_ca_uncoll', 'Uncollectibles'), ('asset_ca_prepaid', 'Prepaid'), ('asset_ca_other', 'Other Liquid Assets'), ('asset_lti_notes', 'Notes Receivable'), ('asset_lti_land', 'Land'), ('asset_lti_sec', 'Securities'), ('asset_ppe_build', 'Buildings'), ('asset_ppe_build_accum_depr', 'Buildings - Accum. Depreciation'), ('asset_ppe_plant', 'Plant'), ('asset_ppe_plant_depr', 'Plant - Accum. Depreciation'), ('asset_ppe_equip', 'Equipment'), ('asset_ppe_equip_accum_depr', 'Equipment - Accum. Depreciation'), ('asset_ia', 'Intangible Assets'), ('asset_ia_accum_amort', 'Intangible Assets - Accum. Amortization'), ('asset_adjustment', 'Other Assets'))), ('Liabilities', (('lia_cl_acc_payable', 'Accounts Payable'), ('lia_cl_wages_payable', 'Wages Payable'), ('lia_cl_int_payable', 'Interest Payable'), ('lia_cl_taxes_payable', 'Taxes Payable'), ('lia_cl_st_notes_payable', 'Short Term Notes Payable'), ('lia_cl_ltd_mat', 'Current Maturities of Long Tern Debt'), ('lia_cl_def_rev', 'Deferred Revenue'), ('lia_cl_other', 'Other Liabilities'), ('lia_ltl_notes', 'Long Term Notes Payable'), ('lia_ltl_bonds', 'Bonds Payable'), ('lia_ltl_mortgage', 'Mortgage Payable'))), ('Equity', (('eq_capital', 'Capital'), ('eq_stock_common', 'Common Stock'), ('eq_stock_preferred', 'Preferred Stock'), ('eq_adjustment', 'Other Equity Adjustments'), ('eq_dividends', 'Dividends & Distributions to Shareholders'), ('in_operational', 'Operational Income'), ('in_passive', 'Investing/Passive Income'), ('in_interest', 'Interest Income'), ('in_gain_loss', 'Capital Gain/Loss Income'), ('in_other', 'Other Income'), ('cogs_regular', 'Cost of Goods Sold'), ('ex_regular', 'Regular Expense'), ('ex_interest', 'Interest Expense'), ('ex_taxes', 'Tax Expense'), ('ex_capital', 'Capital Expense'), ('ex_depreciation', 'Depreciation Expense'), ('ex_amortization', 'Amortization Expense'), ('ex_other', 'Other Expense'))), ('Root', (('root_coa', 'CoA Root Account'), ('root_assets', 'Assets Root Account'), ('root_liabilities', 'Liabilities Root Account'), ('root_capital', 'Capital Root Account'), ('root_income', 'Income Root Account'), ('root_cogs', 'COGS Root Account'), ('root_expenses', 'Expenses Root Account')))], max_length=30, verbose_name='Account Role'),
        ),
        migrations.AlterUniqueTogether(
            name='accountmodel',
            unique_together={('coa_model', 'role', 'role_default'), ('coa_model', 'code')},
        ),
        migrations.AddIndex(
            model_name='customermodel',
            index=models.Index(fields=['entity_model', 'customer_number'], name='django_ledg_entity__7b3ad8_idx'),
        ),
        migrations.AddIndex(
            model_name='vendormodel',
            index=models.Index(fields=['entity_model', 'vendor_number'], name='django_ledg_entity__266aa0_idx'),
        ),
        migrations.AddIndex(
            model_name='vendormodel',
            index=models.Index(fields=['vendor_number'], name='django_ledg_vendor__5f9f46_idx'),
        ),
    ]
