{% extends 'django_ledger/layouts/content_layout_1.html' %}
{% load i18n %}
{% load static %}
{% load django_ledger %}

{% block view_content %}
    <div class="box">
        <a class="button is-primary is-outlined"
           href="{% url 'django_ledger:data-import-ofx' entity_slug=view.kwargs.entity_slug %}">
            {% trans 'Import OFX File' %}</a>
        <a class="button is-dark"
           href="{% url 'django_ledger:entity-dashboard' entity_slug=view.kwargs.entity_slug %}">
            {% trans 'Back' %}</a>
        {% data_import_job_list_table %}
    </div>
{% endblock %}