<nav class="breadcrumb" aria-label="breadcrumbs">
    <ul>
        <li>
            <a href="{% url 'django_ledger:dashboard' %}">
        <span class="icon is-small">
          <i class="mdi mdi-view-dashboard-variant" aria-hidden="true"></i>
        </span>
                <span>Dashboard</span>
            </a>
        </li>

        {% if entity_slug %}
            <li>
                <a href="#">
        <span class="icon is-small">
          <i class="fas fa-book" aria-hidden="true"></i>
        </span>
                    <span>Documentation</span>
                </a>
            </li>
            <li>
                <a href="#">
        <span class="icon is-small">
          <i class="fas fa-puzzle-piece" aria-hidden="true"></i>
        </span>
                    <span>Components</span>
                </a>
            </li>
            <li class="is-active">
                <a href="#">
        <span class="icon is-small">
          <i class="fas fa-thumbs-up" aria-hidden="true"></i>
        </span>
                    <span>Breadcrumb</span>
                </a>
            </li>
        {% endif %}
    </ul>
</nav>