# Generated by Django 4.1.5 on 2023-02-04 16:10

from decimal import Decimal
import django.core.validators
from django.db import migrations, models
import django_ledger.models.accounts


class Migration(migrations.Migration):

    dependencies = [
        ('django_ledger', '0002_alter_journalentrymodel_managers_and_more'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='accountmodel',
            name='django_ledg_role_1bff96_idx',
        ),
        migrations.AlterField(
            model_name='accountmodel',
            name='code',
            field=models.CharField(max_length=10, validators=[django_ledger.models.accounts.account_code_validator], verbose_name='Account Code'),
        ),
        migrations.AlterField(
            model_name='accountmodel',
            name='role',
            field=models.CharField(choices=[('Assets', (('asset_ca_cash', 'Current Asset'), ('asset_ca_mkt_sec', 'Marketable Securities'), ('asset_ca_recv', 'Receivables'), ('asset_ca_inv', 'Inventory'), ('asset_ca_uncoll', 'Uncollectibles'), ('asset_ca_prepaid', 'Prepaid'), ('asset_ca_other', 'Other Liquid Assets'), ('asset_lti_notes', 'Notes Receivable'), ('asset_lti_land', 'Land'), ('asset_lti_sec', 'Securities'), ('asset_ppe_build', 'Buildings'), ('asset_ppe_build_accum_depr', 'Buildings - Accum. Depreciation'), ('asset_ppe_plant', 'Plant'), ('asset_ppe_plant_depr', 'Plant - Accum. Depreciation'), ('asset_ppe_equip', 'Equipment'), ('asset_ppe_equip_accum_depr', 'Equipment - Accum. Depreciation'), ('asset_ia', 'Intangible Assets'), ('asset_ia_accum_amort', 'Intangible Assets - Accum. Amortization'), ('asset_adjustment', 'Other Assets'))), ('Liabilities', (('lia_cl_acc_payable', 'Accounts Payable'), ('lia_cl_wages_payable', 'Wages Payable'), ('lia_cl_int_payable', 'Interest Payable'), ('lia_cl_taxes_payable', 'Taxes Payable'), ('lia_cl_st_notes_payable', 'Notes Payable'), ('lia_cl_ltd_mat', 'Current Maturities of Long Tern Debt'), ('lia_cl_def_rev', 'Deferred Revenue'), ('lia_cl_other', 'Other Liabilities'), ('lia_ltl_notes', 'Notes Payable'), ('lia_ltl_bonds', 'Bonds Payable'), ('lia_ltl_mortgage', 'Mortgage Payable'))), ('Equity', (('eq_capital', 'Capital'), ('eq_stock_common', 'Common Stock'), ('eq_stock_preferred', 'Preferred Stock'), ('eq_adjustment', 'Other Equity Adjustments'), ('eq_dividends', 'Dividends & Distributions to Shareholders'), ('in_operational', 'Operational Income'), ('in_passive', 'Investing/Passive Income'), ('in_interest', 'Interest Income'), ('in_gain_loss', 'Capital Gain/Loss Income'), ('in_other', 'Other Income'), ('cogs_regular', 'Cost of Goods Sold'), ('ex_regular', 'Regular Expense'), ('ex_interest', 'Interest Expense'), ('ex_taxes', 'Tax Expense'), ('ex_capital', 'Capital Expense'), ('ex_depreciation', 'Depreciation Expense'), ('ex_amortization', 'Amortization Expense'), ('ex_other', 'Other Expense'))), ('Root', (('root_coa', 'CoA Root Account'), ('root_assets', 'Assets Root Account'), ('root_liabilities', 'Liabilities Root Account'), ('root_capital', 'Capital Root Account'), ('root_income', 'Income Root Account'), ('root_cogs', 'COGS Root Account'), ('root_expenses', 'Expenses Root Account')))], max_length=30, verbose_name='Account Role'),
        ),
        migrations.AlterField(
            model_name='estimatemodel',
            name='equipment_estimate',
            field=models.DecimalField(decimal_places=2, default=Decimal('0.00'), help_text='Estimated equipment cost to complete the quoted work.', max_digits=20, validators=[django.core.validators.MinValueValidator(limit_value=0)], verbose_name='Equipment Cost Estimate'),
        ),
        migrations.AlterField(
            model_name='estimatemodel',
            name='labor_estimate',
            field=models.DecimalField(decimal_places=2, default=Decimal('0.00'), help_text='Estimated labor cost to complete the quoted work.', max_digits=20, validators=[django.core.validators.MinValueValidator(limit_value=0)], verbose_name='Labor Cost of labor estimate'),
        ),
        migrations.AlterField(
            model_name='estimatemodel',
            name='material_estimate',
            field=models.DecimalField(decimal_places=2, default=0.0, help_text='Estimated material cost to complete the quoted work.', max_digits=20, validators=[django.core.validators.MinValueValidator(limit_value=0)], verbose_name='Material Cost Estimate'),
        ),
        migrations.AlterField(
            model_name='estimatemodel',
            name='other_estimate',
            field=models.DecimalField(decimal_places=2, default=Decimal('0.00'), help_text='Estimated equipment cost to complete the quoted work.', max_digits=20, validators=[django.core.validators.MinValueValidator(limit_value=0)], verbose_name='Other Cost Estimate'),
        ),
        migrations.AlterField(
            model_name='estimatemodel',
            name='revenue_estimate',
            field=models.DecimalField(decimal_places=2, default=Decimal('0.00'), help_text='Estimated cost to complete the quoted work.', max_digits=20, validators=[django.core.validators.MinValueValidator(limit_value=0)], verbose_name='Total revenue estimate'),
        ),
        migrations.AlterField(
            model_name='estimatemodel',
            name='status',
            field=models.CharField(choices=[('draft', 'Draft'), ('in_review', 'In Review'), ('approved', 'Approved'), ('completed', 'Completed'), ('void', 'Void'), ('canceled', 'Canceled')], default='draft', max_length=10, verbose_name='Estimate Model Status'),
        ),
        migrations.AlterField(
            model_name='estimatemodel',
            name='title',
            field=models.CharField(max_length=250, validators=[django.core.validators.MinLengthValidator(limit_value=5, message='EstimateModel Title length must be greater than 5')], verbose_name='Customer Estimate Title'),
        ),
        migrations.AddIndex(
            model_name='accountmodel',
            index=models.Index(fields=['locked'], name='django_ledg_locked_c02b15_idx'),
        ),
    ]
