# Generated by Django 4.1.5 on 2023-02-10 12:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_ledger', '0003_remove_accountmodel_django_ledg_role_1bff96_idx_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='itemmodel',
            name='depth',
        ),
        migrations.RemoveField(
            model_name='itemmodel',
            name='numchild',
        ),
        migrations.RemoveField(
            model_name='itemmodel',
            name='path',
        ),
    ]
