{% extends 'django_ledger/layouts/content_layout_1.html' %}
{% load i18n %}
{% load static %}
{% load django_ledger %}

{% block view_content %}
    <div class="container">
        <div class="columns">
            <div class="column">
                <div class="column is-12">
                    <div class="box">
                        <h1 class="is-size-1 has-text-weight-light has-text-centered">Inventory Ordered</h1>
                        {% if inventory_ordered %}
                            {% inventory_table inventory_ordered %}
                        {% else %}
                            <h1>No inventory in ordered status.</h1>
                        {% endif %}
                    </div>
                </div>
                <div class="column is-12">
                    <div class="box">
                        <h1 class="is-size-1 has-text-weight-light has-text-centered">Inventory In Transit</h1>
                        <div class="table-container">
                            {% if inventory_in_transit %}
                                {% inventory_table inventory_in_transit %}
                            {% else %}
                                <h1>No inventory in transit.</h1>
                            {% endif %}
                        </div>
                    </div>
                </div>
                <div class="column is-12">
                    <div class="box">
                        <h1 class="is-size-1 has-text-weight-light has-text-centered">Inventory Received</h1>
                        <div class="table-container">
                            {% if inventory_received %}
                                {% inventory_table inventory_received %}
                        {% else %}
                            <h1>No inventory in received status.</h1>
                        {% endif %}
                        </div>
                    </div>
                </div>
                <div class="column is-12">
                    <a href="{% url 'django_ledger:inventory-recount' entity_slug=view.kwargs.entity_slug %}"
                       class="button is-info is-small">Inventory Recount</a>
                </div>
            </div>
        </div>
    </div>
{% endblock %}