# Generated by Django 4.1.4 on 2023-01-09 17:12

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_ledger', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='journalentrymodel',
            managers=[
            ],
        ),
        migrations.AlterField(
            model_name='transactionmodel',
            name='amount',
            field=models.DecimalField(decimal_places=2, default=0.0, help_text='Account of the transaction.', max_digits=20, validators=[django.core.validators.MinValueValidator(0)], verbose_name='Amount'),
        ),
    ]
