{% extends 'django_ledger/layouts/content_layout_1.html' %}
{% load i18n %}
{% load static %}

{% block view_content %}
    <div class="columns is-centered">
        <div class="column is-6">
            <div class="box">
                <form method="post">
                    {% csrf_token %}
                    {{ form.as_p }}
                    <button class="button is-fullwidth is-primary is-outlined djetler_my_1" type="submit">Update
                    </button>
                    <a href="{{ journal_entry.get_journal_entry_list_url }}"
                       class="button is-small is-dark is-fullwidth">Back</a>
                </form>
            </div>
        </div>
    </div>
{% endblock %}