{% extends 'django_ledger/layouts/content_layout_1.html' %}
{% load i18n %}
{% load static %}

{% block view_content %}
    <div class="columns is-centered is-multiline">
        <div class="column is-12 has-text-centered">
            <h1 class="is-size-1">{% trans 'Create Journal Entry' %}</h1>
            <h2 class="is-size-4">{{ ledger_model.name }}</h2>
        </div>
        <div class="column is-4">
            <div class="box">
                <form method="post">
                    {% csrf_token %}
                    {{ form.as_p }}
                    <button class="button is-outlined is-primary is-fullwidth djetler_my_1" type="submit">Create
                    </button>
                    <a class="button is-small is-dark is-fullwidth"
                       href="{{ ledger_model.get_journal_entry_list_url }}">Back</a>
                </form>
            </div>
        </div>
    </div>
{% endblock %}
