{% extends 'django_ledger/layouts/content_layout_1.html' %}
{% load i18n %}
{% load static %}

{% block view_content %}
    <div class="columns is-centered">
        <div class="column is-8-tablet is-6-desktop">
            <div class="box">
                <form action="{{ coa_model.get_update_url }}" id="{{ form.form_id }}" method="post">
                    {% csrf_token %}
                    {{ form.as_p }}
                    <button class="button is-primary is-outlined is-fullwidth djetler_my_1"
                            type="submit">Update
                    </button>
                    <a class="button is-small is-dark is-fullwidth"
                       href="{{ coa_model.get_coa_list_url }}"
                       type="submit">Back</a>
                </form>
            </div>
        </div>
    </div>
{% endblock %}