{% extends 'django_ledger/layouts/content_layout_1.html' %}
{% load i18n %}
{% load static %}
{% load django_ledger %}

{% block aux_menu %}
    {% period_navigation 'account-detail' %}
{% endblock %}

{% block view_content %}
    <div class="card">
        <div class="card-content has-text-centered">
            <h1 class="is-size-1 has-text-weight-bold">{{ account.code }}: {{ account.name }}</h1>
            <h2 class="is-size-2 has-text-weight-light">{% trans 'Account Transaction List Report' %}</h2>
            <h3 class="is-size-4 is-italic has-font-weight-light">
                {% if view_date %}
                    {{ from_date | date:'m/d/Y' }}
                {% else %}
                    {{ from_date | date:'m/d/Y' }} - {{ to_date | date:'m/d/Y' }}
                {% endif %}
            </h3>
        </div>
        <div class="card-content">{% account_txs_table transactions %}</div>
    </div>
{% endblock %}