{% extends 'django_ledger/layouts/content_layout_1.html' %}
{% load i18n %}
{% load static %}
{% load django_ledger %}

{% block view_content %}

    <div class="box">
        <div class="columns">
            <div class="column">
                <a class="button is-primary is-outlined is-small"
                   href="{% url 'django_ledger:bank-account-create' entity_slug=view.kwargs.entity_slug %}">
                    <span class="icon is-large has-text-success">{% icon 'carbon:add-alt' 24 %}</span>
                    <span>{% trans 'New Bank Account' %}</span>
                </a>
            </div>
        </div>
        {% bank_account_table bank_accounts %}
    </div>
{% endblock %}
