# Generated by Django 4.1.5 on 2023-02-15 14:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_ledger', '0004_remove_itemmodel_depth_remove_itemmodel_numchild_and_more'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='itemmodel',
            name='django_ledg_invento_dbf206_idx',
        ),
        migrations.RemoveIndex(
            model_name='itemmodel',
            name='django_ledg_cogs_ac_82d441_idx',
        ),
        migrations.RemoveIndex(
            model_name='itemmodel',
            name='django_ledg_earning_229a60_idx',
        ),
        migrations.RemoveIndex(
            model_name='itemmodel',
            name='django_ledg_expense_f65128_idx',
        ),
        migrations.RemoveIndex(
            model_name='itemmodel',
            name='django_ledg_item_id_972760_idx',
        ),
        migrations.AddField(
            model_name='itemmodel',
            name='item_role',
            field=models.CharField(blank=True, choices=[('expense', 'Expense'), ('inventory', 'Inventory'), ('service', 'Service'), ('product', 'Product')], max_length=10, null=True),
        ),
        migrations.AlterField(
            model_name='journalentrymodel',
            name='timestamp',
            field=models.DateTimeField(verbose_name='Timestamp'),
        ),
        migrations.AddIndex(
            model_name='itemmodel',
            index=models.Index(fields=['item_role'], name='django_ledg_item_ro_6040aa_idx'),
        ),
    ]
