{% extends 'django_ledger/layouts/base.html' %}
{% load trans from i18n %}
{% load static %}

{% block nav %}{% endblock %}
{% block header %}{% endblock %}
{% block content %}
    <section class="hero is-fullheight"
             id="djl-login-bg-image"
             style="background-image: url({% static 'django_ledger/img/daniel-weiss-aj2Os9mYgJU-unsplash.jpg' %})">
        <div class="hero-body">
            <div class="container is-fluid">
                <div class="columns is-centered">
                    <div class="column is-6-tablet is-4-desktop">
                        <div class="box">
                            <div class="container" style="padding: 20px;">
                                <div class="figure">
                                    <img src="{% static 'django_ledger/logo/django-ledger-logo.png' %}" alt="">
                                </div>
                            </div>
                            <form action="{% url 'django_ledger:login' %}" method="post" id="djl-el-login-form">
                                {% csrf_token %}
                                <div class="columns is-multiline">
                                    <div class="column is-12">{{ form }}</div>
                                    <div class="column is-12">
                                        <button type="submit"
                                                class="button is-primary is-fullwidth">
                                            {% trans 'Login' %}
                                        </button>
                                    </div>
                                </div>
                            </form>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </section>
{% endblock %}
{% block footer %}{% endblock %}