{% extends 'django_ledger/layouts/base.html' %}
{% load i18n %}
{% load static %}

{% block content %}

    <section class="section">
        <div class="container">
            <div class="columns is-centered">
                <div class="column is-4">
                    <form method="post"
                          action="{{ entity.get_update_url }}">
                        {% csrf_token %}
                        {{ form.as_p }}
                        <button class="button is-fullwidth is-primary is-outlined is-dark djetler_my_1"
                                type="submit">{% trans 'Update' %}
                        </button>
                        <a class="button is-small is-fullwidth is-dark"
                           href="{% url 'django_ledger:entity-dashboard' entity_slug=view.kwargs.entity_slug %}">
                            {% trans 'Back' %}
                        </a>
                    </form>

                </div>
            </div>
        </div>

    </section>

{% endblock %}
