{% load django_ledger %}
<div class="table-container">
    <table class="table is-fullwidth is-narrow is-striped is-bordered">
        <thead>
        <tr class="has-text-centered">
            <th>Item</th>
            <th>UOM</th>
            <th>Quantity</th>
            <th>Value</th>
        </tr>
        </thead>
        <tbody>
        {% for i in inventory_list %}
            <tr>
                <td>{{ i.item_model__name }}</td>
                <td>{{ i.item_model__uom__name }}</td>
                <td>{{ i.total_quantity | floatformat:3 }}</td>
                <td>{% currency_symbol %}{{ i.total_value | currency_format }}</td>
            </tr>
        {% endfor %}
        </tbody>
        <tfoot>
        <tr>
            <th></th>
            <th></th>
            <th>Total Value</th>
            <th>{% currency_symbol %}{{ inventory_total_value | currency_format }}</th>
        </tr>
        </tfoot>
    </table>
</div>