{% if messages %}
    {# Messages Container #}
    <div class="container px-2">
        {% for msg in messages %}
            <div class="notification {{ msg.tags }} mt-3 mb-2">
                <button class="delete"></button>
                {{ msg | safe }}
            </div>
        {% endfor %}
    </div>
{% endif %}