# Generated by Django 5.2.1 on 2025-08-22 19:05

import django.db.models.deletion
import django_ledger.models.customer
import django_ledger.models.vendor
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_ledger', '0022_bankaccountmodel_financial_institution_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='customermodel',
            name='customer_code',
            field=models.SlugField(blank=True, null=True, unique=True, verbose_name='User defined customer code'),
        ),
        migrations.AddField(
            model_name='customermodel',
            name='picture',
            field=models.ImageField(blank=True, null=True, upload_to=django_ledger.models.customer.customer_picture_upload_to),
        ),
        migrations.AddField(
            model_name='vendormodel',
            name='picture',
            field=models.ImageField(blank=True, null=True, upload_to=django_ledger.models.vendor.vendor_picture_upload_to),
        ),
        migrations.AddField(
            model_name='vendormodel',
            name='vendor_code',
            field=models.SlugField(blank=True, null=True, verbose_name='User defined vendor code.'),
        ),
        migrations.AlterField(
            model_name='customermodel',
            name='customer_number',
            field=models.CharField(editable=False, help_text='System generated customer number.', max_length=30, verbose_name='Customer Number'),
        ),
        migrations.AlterField(
            model_name='vendormodel',
            name='entity_model',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to='django_ledger.entitymodel', verbose_name='Vendor Entity'),
        ),
        migrations.AlterField(
            model_name='vendormodel',
            name='vendor_number',
            field=models.CharField(blank=True, editable=False, help_text='System generated vendor number.', max_length=30, null=True, verbose_name='Vendor Number'),
        ),
    ]
