.djetler_my_1 {
  margin-top: 1rem;
  margin-bottom: 1rem;
}
.djetler_mt_1 {
  margin-top: 1rem;
}
.djetler_mb_1 {
  margin-bottom: 1rem;
}
.djetler_my_2 {
  margin-top: 2rem;
  margin-bottom: 2rem;
}
.djetler_mt_2 {
  margin-top: 2rem;
}
.djetler_mb_2 {
  margin-bottom: 2rem;
}
.djetler_my_3 {
  margin-top: 3rem;
  margin-bottom: 3rem;
}
.djetler_mt_3 {
  margin-top: 3rem;
}
.djetler_mb_3 {
  margin-bottom: 3rem;
}
.djetler_my_4 {
  margin-top: 4rem;
  margin-bottom: 4rem;
}
.djetler_mt_4 {
  margin-top: 4rem;
}
.djetler_mb_4 {
  margin-bottom: 4rem;
}
.djetler_my_5 {
  margin-top: 5rem;
  margin-bottom: 5rem;
}
.djetler_mt_5 {
  margin-top: 5rem;
}
.djetler_mb_5 {
  margin-bottom: 5rem;
}
.djetler_my_6 {
  margin-top: 6rem;
  margin-bottom: 6rem;
}
.djetler_mt_6 {
  margin-top: 6rem;
}
.djetler_mb_6 {
  margin-bottom: 6rem;
}
.djetler_my_7 {
  margin-top: 7rem;
  margin-bottom: 7rem;
}
.djetler_mt_7 {
  margin-top: 7rem;
}
.djetler_mb_7 {
  margin-bottom: 7rem;
}
.djetler_my_8 {
  margin-top: 8rem;
  margin-bottom: 8rem;
}
.djetler_mt_8 {
  margin-top: 8rem;
}
.djetler_mb_8 {
  margin-bottom: 8rem;
}
.djetler_my_9 {
  margin-top: 9rem;
  margin-bottom: 9rem;
}
.djetler_mt_9 {
  margin-top: 9rem;
}
.djetler_mb_9 {
  margin-bottom: 9rem;
}
.djetler_my_10 {
  margin-top: 10rem;
  margin-bottom: 10rem;
}
.djetler_mt_10 {
  margin-top: 10rem;
}
.djetler_mb_10 {
  margin-bottom: 10rem;
}
.djetler_mx_1 {
  margin-left: 1rem;
  margin-right: 1rem;
}
.djetler_ml_1 {
  margin-left: 1rem;
}
.djetler_mr_1 {
  margin-right: 1rem;
}
.djetler_mx_2 {
  margin-left: 2rem;
  margin-right: 2rem;
}
.djetler_ml_2 {
  margin-left: 2rem;
}
.djetler_mr_2 {
  margin-right: 2rem;
}
.djetler_mx_3 {
  margin-left: 3rem;
  margin-right: 3rem;
}
.djetler_ml_3 {
  margin-left: 3rem;
}
.djetler_mr_3 {
  margin-right: 3rem;
}
.djetler_mx_4 {
  margin-left: 4rem;
  margin-right: 4rem;
}
.djetler_ml_4 {
  margin-left: 4rem;
}
.djetler_mr_4 {
  margin-right: 4rem;
}
.djetler_mx_5 {
  margin-left: 5rem;
  margin-right: 5rem;
}
.djetler_ml_5 {
  margin-left: 5rem;
}
.djetler_mr_5 {
  margin-right: 5rem;
}
.djetler_mx_6 {
  margin-left: 6rem;
  margin-right: 6rem;
}
.djetler_ml_6 {
  margin-left: 6rem;
}
.djetler_mr_6 {
  margin-right: 6rem;
}
.djetler_mx_7 {
  margin-left: 7rem;
  margin-right: 7rem;
}
.djetler_ml_7 {
  margin-left: 7rem;
}
.djetler_mr_7 {
  margin-right: 7rem;
}
.djetler_mx_8 {
  margin-left: 8rem;
  margin-right: 8rem;
}
.djetler_ml_8 {
  margin-left: 8rem;
}
.djetler_mr_8 {
  margin-right: 8rem;
}
.djetler_mx_9 {
  margin-left: 9rem;
  margin-right: 9rem;
}
.djetler_ml_9 {
  margin-left: 9rem;
}
.djetler_mr_9 {
  margin-right: 9rem;
}
.djetler_mx_10 {
  margin-left: 10rem;
  margin-right: 10rem;
}
.djetler_ml_10 {
  margin-left: 10rem;
}
.djetler_mr_10 {
  margin-right: 10rem;
}
.djetler-section-min100 {
  min-height: 100px;
}
.djetler-section-min200 {
  min-height: 200px;
}
.djetler-section-min300 {
  min-height: 300px;
}
.djetler-section-min400 {
  min-height: 400px;
}
.djetler-section-min500 {
  min-height: 500px;
}
.djetler-box-green {
  background-color: #18ff6f;
}
.djetler-box-yellow {
  background-color: #ffd931;
}
.djetler-box-blue {
  background-color: #374bff;
}
.django-ledger-login {
  height: 100vh;
  background-size: cover;
  background-position: center;
}
.djl-icon-12 {
  font-size: 12px;
}
.djl-icon-24 {
  font-size: 24px;
}
.djl-icon-36 {
  font-size: 36px;
}
.djl-icon-48 {
  font-size: 48px;
}
.djl-icon-60 {
  font-size: 60px;
}
.djl-icon-72 {
  font-size: 72px;
}
.djl-icon-84 {
  font-size: 84px;
}
.djl-icon-96 {
  font-size: 96px;
}
.djl-icon-108 {
  font-size: 108px;
}
.djl-icon-120 {
  font-size: 120px;
}
.django-ledger-table-bottom-margin-75 {
  margin-bottom: 75px;
}
.django-ledger-table-bottom-margin-150 {
  margin-bottom: 150px;
}
.djl-is-strikethrough {
  text-decoration: line-through;
}
#djl-login-bg-image {
  background-repeat: no-repeat;
  background-size: cover;
}
/*# sourceMappingURL=djl_styles.css.map */