{% extends 'django_ledger/layouts/content_layout_1.html' %}
{% load i18n %}
{% load static %}
{% load django_ledger %}

{% block view_content %}


    <div class="columns is-multiline is-centered">
        {% if estimate_model %}
            <div class="column is-12 has-text-centered">
                <h2 class="is-size-2">{% trans 'PO for Estimate' %}: {{ estimate_model.title }}</h2>
                <h3 class="is-size-3">{{ estimate_model.estimate_number }}</h3>
                <h4 class="is-size-4 mt-2">{{ estimate_model.customer }}</h4>
                <a class="button is-dark mt-2"
                   href="{% url 'django_ledger:customer-estimate-detail' entity_slug=view.kwargs.entity_slug ce_pk=estimate_model.uuid %}">
                    {% trans 'Back to Estimate' %}</a>
            </div>
        {% endif %}

        <div class="column is-6">
            <form action="{{ form_action_url }}" method="post">
                {% csrf_token %}
                {{ form }}
                <button type="submit"
                        class="button is-primary is-outlined is-fullwidth djetler_my_1">
                    {% trans 'Create' %}
                </button>
            </form>
        </div>
    </div>
{% endblock %}