# Generated by Django 4.2.2 on 2023-09-22 20:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_ledger', '0009_delete_stagedtransactionmodel'),
    ]

    operations = [
        migrations.DeleteModel(
            name='ImportJobModel',
        ),
    ]
