<footer class="footer">
    <div class="content has-text-centered">
        <p>
            <strong>Django Ledger</strong> by <a href="https://www.miguelsanda.com" target="_blank">Miguel Sanda</a> and
            <a href="https://miguelsanda.com" target="_blank">EDMA Group Inc</a>. The source code is licensed
            <a href="https://opensource.org/licenses/GPL-3.0" target="_blank">GPL 3.0</a>
        </p>
        {% if user.is_superuser %}
            <a href="/admin">Admin</a>
        {% endif %}
    </div>
</footer>